@@ -11,7 +11,7 @@
 
 namespace Botan {
 
-class BOTAN_DLL EME_Raw : public EME
+class BOTAN_DLL EME_Raw final : public EME
    {
    public:
       size_t maximum_input_size(size_t i) const override;
@@ -21,7 +21,9 @@ class BOTAN_DLL EME_Raw : public EME
       secure_vector<byte> pad(const byte[], size_t, size_t,
                              RandomNumberGenerator&) const override;
 
-      secure_vector<byte> unpad(const byte[], size_t, size_t) const override;
+      secure_vector<byte> unpad(byte& valid_mask,
+                                const byte in[],
+                                size_t in_len) const override;
    };
 
 }
