@@ -21,6 +21,8 @@ class BOTAN_DLL RC4 final : public StreamCipher
    public:
       void cipher(const byte in[], byte out[], size_t length) override;
 
+      void set_iv(const byte iv[], size_t iv_len) override;
+
       void clear() override;
       std::string name() const override;
 
@@ -31,14 +33,14 @@ class BOTAN_DLL RC4 final : public StreamCipher
          return Key_Length_Specification(1, 256);
          }
 
-      static RC4* make(const Spec& spec);
-
       /**
       * @param skip skip this many initial bytes in the keystream
       */
       explicit RC4(size_t skip = 0);
 
       ~RC4() { clear(); }
+
+      void seek(u64bit offset) override;
    private:
       void key_schedule(const byte[], size_t) override;
       void generate();
